<?php

/**
 * @file
 * Drush integration for block instances module.
 */

/* @TODO
 * Allow to move the block defined in a block instance from a region to another region
 * Allow to hide all the blocks defined in a region
 */

/**
 * Adds a cache clear option for block instances.
 */
function block_instances_drush_cache_clear(&$types) {
  $types['block_instances'] = 'block_instances_invalidate_cache';
}

/**
 * Implementation of hook_drush_help().
 */
function block_instances_drush_help($section) {
  switch ($section) {
    case 'drush:block-instances-list':
      return dt('List all block instances.');
    case 'drush:block-instance-enable':
      return dt('Enable a disabled block instance.');
    case 'drush:block-instance-disabled':
      return dt('Disable an enabled block instance.');
  }
}

/**
 * Implementation of hook_drush_command().
 */
function block_instances_drush_command() {
  $items['block-instances-list'] = array(
    'callback' => 'block_instances_drush_list',
    'description' => 'List all block instances.',
    'aliases' => array('bil'),
  );
  $items['block-instance-enable'] = array(
    'callback' => 'block_instances_drush_enable',
    'description' => 'Enable a disabled block instance.',
    'aliases' => array('enbi'),
  );
  $items['block-instance-disable'] = array(
    'callback' => 'block_instances_drush_disable',
    'description' => 'Disable an enabled block instance.',
    'aliases' => array('disbi'),
  );
  return $items;
}

/**
 * Implementation of 'block instances list' command.
 */
function block_instances_drush_list() {
  $block_instances = block_instances_crud_load_all();

  foreach (module_implements('block') as $module) {
    foreach (module_invoke($module, 'block', 'list') as $id => $block) {
      $blocks["$module:$id"] = filter_xss($block['info']);
    }
  }

  $rows = array();
  $rows[] = array('Block instance', 'Block', 'Region', 'Status');
  $rows[] = array('--------------', '-----', '------', '------');

  $status = array(
    1 => t('Disabled'),
    0 => t('Enabled'),
  );
  foreach ($block_instances as $block_instance) {
    if (!empty($last_region) && ($last_region != $block_instance->region)) {
      $rows[] = array('', '', '', '');
    }
    $last_region = $block_instance->region;
    $rows[] = array($block_instance->admin_title . "  ($block_instance->machine_name)",
                    $blocks[$block_instance->settings['block']],
                    $block_instance->region,
                    !empty($block_instance->disabled) ? t('Disabled') : t('Enabled'),
              );
  }
  drush_print_table($rows, TRUE);
}

/**
 * Enable a block instance
 */
function block_instances_drush_enable() {
  $args = func_get_args();
  block_instances_drush_set_status('enable', $args);
}

/**
 * Disable a block instance
 */
function block_instances_drush_disable() {
  $args = func_get_args();
  block_instances_drush_set_status('disable', $args);
}

function block_instances_drush_set_status($status, $args) {
  ctools_include('export');
  foreach ($args as $block_instance_name) {
    if ($block_instance = block_instances_crud_load($block_instance_name)) {
      ctools_export_set_object_status($block_instance, $status != 'enable');
      if ($status == 'enable') {
        drush_print(dt('The block instance !bi has been enabled.', array('!bi' => $block_instance->admin_title)));
      }
      else {
        drush_print(dt('The block instance !bi has been disabled.', array('!bi' => $block_instance->admin_title)));
      }
    }
    else {
      drush_print(dt('The block instance !bi does not exists.', array('!bi' => $block_instance_name)));
    }
  }
  block_instances_invalidate_cache();
}
